#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 18 09:27:48 2017

@author: robin
"""

from sklearn.base import BaseEstimator, ClassifierMixin
from fastText import load_model
import pkg_resources


def is_addr(fasttext_output):
    """
    the fasttext model is:
    '__label__1': an address
    '__label__0': not an address
    """
    if fasttext_output:
        return fasttext_output[0] == '__label__1'
    return False


class Fasttext_clf(BaseEstimator, ClassifierMixin):
    data_path = pkg_resources.resource_filename('addr_detector.model', 'ft_ad.ftz')

    def __init__(self, path=data_path):
        self.model = load_model(path)

    def fit(self, X, y):
        return self

    def predict(self, X):
        r = self.model.predict(X)
        if r:
            return is_addr(r[0])
        return False

    def predict_proba(self, X):
        r = self.model.predict(X)
        if r and len(r) >= 2:
            probas = r[1]
            if probas:
                return probas[0]
        return False


def score(self, X, y=None):
    return sum(self.predict(X))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `addr_detector` package."""

import pytest
from addr_detector.Fasttext_clf import Fasttext_clf
from addr_detector.Postal_clf import Postal_clf


@pytest.fixture
def postal():
    return Postal_clf()


@pytest.fixture
def fasttext():
    return Fasttext_clf()


def test_good_addr_fasttext(fasttext):
    assert fasttext.predict('7 rue spontini paris')


def test_bad_addr_fasttext(fasttext):
    assert not fasttext.predict('Comment se faire cuire un oeuf')


def test_good_addr_postal(postal):
    assert postal.predict('7 rue spontini paris')


def test_bad_addr_postal(postal):
    assert not postal.predict('Comment se faire cuire un oeuf')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 18 16:29:59 2017

@author: robin
"""

from sklearn.ensemble import VotingClassifier
from .Postal_clf import Postal_clf
from .Fasttext_clf import Fasttext_clf

from sklearn.base import BaseEstimator, ClassifierMixin


class addr_voting(BaseEstimator, ClassifierMixin):
    postal_clf = Postal_clf()
    # Temporary, replace with sk_ft() when issue is fixed.
    ft_clf = Fasttext_clf()

    def __init__(self, classifiers=[('postal', postal_clf), ('ft', ft_clf)]):
        self.model = VotingClassifier(estimators=classifiers)

    def fit(self, X, y):
        self.model.fit(X, y)
        return self

    def predict(self, X):
        results = []
        if isinstance(X, str):  #
            results = results + [self.model.predict(X)]
        elif isinstance(X, list):
            results = results + self.model.predict(X)
        return results

    def predict_proba(self, X):
        results = []
        if isinstance(X, str):  #
            results = results + [self.model.predict(X)]
        elif isinstance(X, list):
            results = results + self.model.predict(X)
        return results


def score(self, X, y=None):
    return sum(self.predict(X))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 18 09:27:48 2017

@author: robin
"""

from sklearn.base import BaseEstimator, ClassifierMixin
from postal.parser import parse_address
from postal.expand import expand_address


class Postal_clf(BaseEstimator, ClassifierMixin):
    def __init__(
            self,
            param=None,
            expander=expand_address,
            parser=parse_address):
        self.param = None
        self.parser = parser
        self.expander = expander

    def fit(self, X, y):
        print(2)
        return self

    def predict(self, X):
        return self.is_addr(X)

    def is_addr(self, X):
        full_road = False
        exp_add = self.expander(X)
        parsed_add = self.parser(exp_add[0])
        num_elt_addr = len(parsed_add)
        address = dict.fromkeys(['house_number',
                                 'road',
                                 'house',
                                 'suburb',
                                 'city_district',
                                 'city',
                                 'state',
                                 'postcode',
                                 'country'])

        for x, y in parsed_add:
            address[y] = x
        if address['road'] is not None:
            # len(query_road_set)>1 # We check the length of the troad.  If
            # eq=1, means it's just dummy name (like street)
            full_road = True
        if num_elt_addr >= 4:
            return True

        elif (address['house_number'] is not None and address['city'] is not None and full_road):
            return True
        elif (address['house_number'] is not None and address['postcode'] is not None and full_road):
            return True
        elif (address['house_number'] is not None and full_road):
            return False
        elif ((address['postcode'] or address['city']) is not None and full_road):
            return True
        return False

    def score(self, X, y=None):
        return sum(self.predict(X))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    # TODO: put package requirements here
    'Cython',
    'numpy',
    'scipy',
    'postal==1.0',
    'scikit_learn==0.19',
]

setup_requirements = [
    # TODO(rdoume): put setup requirements (distutils extensions, etc.) here
    'bumpversion==0.5',
    'wheel==0.29',
    'watchdog==0.8',
    'flake8==2.6',
    'Cython',
    'tox==2.3',
    'coverage==4.1',
    'Sphinx==1.4',
    'pytest-runner'
]

test_requirements = [
    # TODO: put package test requirements here
    'pytest==3.2.3'
]

class CustomGitDependenciesInstallCommand(install):
    """
    setuptool does not seems to be able to install git dependencies (with dependency_links)
    so we use a pip install by hand
    """

    def run(self):
        c = ['pip',
             'install',
             'scipy', # numpy and scipy does not seems to be installed too
             'numpy',
             'git+git://github.com/facebookresearch/fastText.git@c5cb6b2d0e295e58cfa827e38d2e9b1e0cbe09e4#egg=fastTextpy&subdirectory=python']
        subprocess.check_call(c)
        install.run(self)

setup(
    cmdclass={'install': CustomGitDependenciesInstallCommand},
    name='addr_detector',
    version='0.2.0',
    description="Python address detector ",
    long_description=readme + '\n\n' + history,
    author="Qwant",
    author_email='robin@qwantresearch.com',
    url='https://github.com/rdoume/addr_detector',
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.ftz']},
    data_files=[('addr_detector.model', ['addr_detector/model/ft_ad.ftz'])],
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='addr_detector',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.5',

    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements
)
